mod directive;
mod event;
mod functions;
mod helpers;
mod state;

pub use directive::*;
pub use event::*;
pub use functions::OutfitState;
pub use helpers::{anchor::*, color::*, rect::*, vector::*};
pub use state::*;

use super::UiState;
use galactica_content::Content;
use galactica_system::PlayerDirective;
use glyphon::FontSystem;
use rhai::{exported_module, Engine};
use std::{cell::RefCell, rc::Rc, sync::Arc};

pub fn register_into_engine(
	engine: &mut Engine,
	ct_src: Arc<Content>,
	font_src: Rc<RefCell<FontSystem>>,
	state_src: Rc<RefCell<UiState>>,
) {
	engine
		// Helpers
		.build_type::<Rect>()
		.build_type::<Color>()
		.build_type::<UiVector>()
		// State
		.build_type::<State>()
		.build_type::<ShipState>()
		.build_type::<SystemObjectState>()
		.build_type::<OutfitState>()
		// Events
		.build_type::<MouseClickEvent>()
		.build_type::<MouseHoverEvent>()
		.build_type::<PlayerShipStateEvent>()
		.build_type::<KeyboardEvent>()
		.build_type::<ScrollEvent>()
		// Enums
		.register_type_with_name::<Anchor>("Anchor")
		.register_static_module("Anchor", exported_module!(anchor_mod).into())
		.register_type_with_name::<PlayerDirective>("PlayerDirective")
		.register_static_module(
			"PlayerDirective",
			exported_module!(player_directive_module).into(),
		);

	// Extra functions
	engine.register_fn("clamp", |x: f32, l: f32, h: f32| x.clamp(l, h));

	// Modules
	engine.register_static_module("ui", functions::build_ui_module(state_src.clone()).into());
	engine.register_static_module("ct", functions::build_ct_module(ct_src.clone()).into());
	engine.register_static_module(
		"sprite",
		functions::build_sprite_module(ct_src.clone(), state_src.clone()).into(),
	);
	engine.register_static_module(
		"textbox",
		functions::build_textbox_module(font_src.clone(), state_src.clone()).into(),
	);
	engine.register_static_module(
		"radialbar",
		functions::build_radialbar_module(state_src.clone()).into(),
	);
	engine.register_static_module(
		"scrollbox",
		functions::build_scrollbox_module(state_src.clone()).into(),
	);
	engine.register_static_module(
		"conf",
		functions::build_conf_module(state_src.clone()).into(),
	);
}
